;******************************************************************************/
;*                                                                            */
;*  Cedocida DV-Codec                                                         */
;*                                                                            */
;*  Copyright (C) 2005 Andreas Dittrich <dittrich@cithraidt.de>               */
;*  Project start: 15.12.2003                                                 */
;*  Last change  : 08.05.2005                                                 */
;*                                                                            */
;******************************************************************************/
;*                                                                            */
;*  This program is free software; you can redistribute it and/or modify      */
;*  it under the terms of the GNU General Public License as published by      */
;*  the Free Software Foundation; either version 2 of the License, or         */
;*  (at your option) any later version.                                       */
;*                                                                            */
;*  This program is distributed in the hope that it will be useful,           */
;*  but WITHOUT ANY WARRANTY; without even the implied warranty of            */
;*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             */
;*  GNU General Public License for more details.                              */
;*                                                                            */
;*  You should have received a copy of the GNU General Public License         */
;*  along with this program; if not, write to the Free Software               */
;*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
;*                                                                            */
;******************************************************************************/

; modified version from Avisynth v2.5., http://www.avisynth.org

%include "../nasm.inc"

;=============================================================================
; Read only data
;=============================================================================

SECTION .rodata align=16

ALIGN 64
yuv2rgb_constants:
yuv2rgb_constants_rec601:
dd	0x00100010, 0x00000000
dd	0x00800080, 0x00800080
dd	0x00FF00FF, 0x00FF00FF
dd	0x00002000, 0x00002000
dd	0xFF000000, 0xFF000000
dd	0x00004A85, 0x00004A85	;cy			; 19077, 19077						; 255/219 << 14
dd	0x33130000, 0x33130000	;crv		; 13075, 13075						; (1-0.299)*255/224 << 14 = 13074.65
dd	0xE5FCF377, 0xE5FCF377	;cgu_cgv	; -6660, -3209, -6660, -3209		; -0.35706814310051*255/224 << 14, -0.17206814310051*255/224 << 14
dd	0x0000408D, 0x0000408D	;cbu		; 16525, 16525						; (1-0.114)*255/224 << 14 = 16525.17

yuv2rgb_constants_rec709:
dd	0x00100010, 0x00000000		; 0, 0, 16, 16
dd	0x00800080, 0x00800080		; 128, 128, 128, 128
dd	0x00FF00FF, 0x00FF00FF		; 255, 255, 255, 255
dd	0x00002000, 0x00002000		; 2^13, 2^13
dd	0xFF000000, 0xFF000000		; 255*2^24, 255*2^24
dd	0x00004A85, 0x00004A85		; 19077, 19077
dd	0x39600000, 0x39600000		; 14688*2^16, 14688*2^16
dd	0xEEF5F930, 0xEEF5F930		; 
dd	0x0000439B, 0x0000439B		; 17307, 17307

yuv2rgb_constants_rec601_full:
dd	0x00000000, 0x00000000		; 0, 0, 0, 0
dd	0x00800080, 0x00800080		; 128, 128, 128, 128
dd	0x00FF00FF, 0x00FF00FF		; 255, 255, 255, 255
dd	0x00002000, 0x00002000		; 2^13, 2^13
dd	0xFF000000, 0xFF000000		; 255*2^24, 255*2^24
dd	0x00004000, 0x00004000		; 16384, 16384						; 1 << 14
dd	0x2CDD0000, 0x2CDD0000		; 11485, 11485						; (1-0.299) << 14
dd	0xE926F4FD, 0xE926F4FD		; -5850, -2819, -5850, -2819		; -0.35706814310051 << 14, -0.17206814310051 << 14
dd	0x000038B4, 0x000038B4		; 14516, 14516

%define ofs_x0000_0000_0010_0010   0
%define ofs_x0080_0080_0080_0080   8
%define ofs_x00FF_00FF_00FF_00FF   16
%define ofs_x00002000_00002000   24
%define ofs_xFF000000_FF000000   32
%define ofs_cy   40
%define ofs_crv   48
%define ofs_cgu_cgv   56
%define ofs_cbu   64


;=============================================================================
; Code
;=============================================================================

SECTION .text


%macro GET_Y 2
	%if %2
		psrlw		%1, 8
	%else
		pand		%1, [rdx+ofs_x00FF_00FF_00FF_00FF]
	%endif
%endmacro


%macro GET_UV 2
	GET_Y %1, (1-%2)
%endmacro


%macro YUV2RGB_INNER_LOOP 3

	;; This YUV422->RGB conversion code uses only four MMX registers per
	;; source dword, so I convert two dwords in parallel.  Lines corresponding
	;; to the "second pipe" are indented an extra space.  There's almost no
	;; overlap, except at the end and in the three lines marked ***.
	;; revised 4july,2002 to properly set alpha in rgb32 to default "on" & other small memory optimizations

	movd		mm0,[rsi]
	movd		 mm5,[rsi+4]
	movq		mm1,mm0
	GET_Y		mm0,%1	; mm0 = __________Y1__Y0
	movq		 mm4,mm5
	GET_UV		mm1,%1	; mm1 = __________V0__U0
	GET_Y		 mm4,%1
	movq		mm2,mm5		; *** avoid reload from [rsi+4]
	GET_UV		 mm5,%1
	psubw		mm0,[rdx+ofs_x0000_0000_0010_0010]
	movd		 mm6,[rsi+8-4*(%3)]
	GET_UV		mm2,%1	; mm2 = __________V2__U2
	psubw		 mm4,[rdx+ofs_x0000_0000_0010_0010]
	paddw		mm2,mm1
	GET_UV		 mm6,%1
	psubw		mm1,[rdx+ofs_x0080_0080_0080_0080]
	paddw		 mm6,mm5
	psllq		mm2,32
	psubw		 mm5,[rdx+ofs_x0080_0080_0080_0080]
	punpcklwd	mm0,mm2		; mm0 = ______Y1______Y0
	psllq		 mm6,32
	pmaddwd		mm0,[rdx+ofs_cy]
	punpcklwd	 mm4,mm6
	paddw		mm1,mm1
	pmaddwd	 mm4,[rdx+ofs_cy]
	paddw		 mm5,mm5
	paddw		mm1,mm2		; mm1 = __V1__U1__V0__U0 * 2
	paddd		mm0,[rdx+ofs_x00002000_00002000]
	paddw		 mm5,mm6
	movq		mm2,mm1
	paddd		 mm4,[rdx+ofs_x00002000_00002000]
	movq		mm3,mm1
	movq		 mm6,mm5
	pmaddwd		mm1,[rdx+ofs_crv]
	movq		 mm7,mm5
	paddd		mm1,mm0
	pmaddwd	 mm5,[rdx+ofs_crv]
	psrad		mm1,14		; mm1 = RRRRRRRRrrrrrrrr
	paddd		 mm5,mm4
	pmaddwd		mm2,[rdx+ofs_cgu_cgv]
	psrad		 mm5,14
	paddd		mm2,mm0
	pmaddwd	 mm6,[rdx+ofs_cgu_cgv]
	psrad		mm2,14		; mm2 = GGGGGGGGgggggggg
	paddd		 mm6,mm4
	pmaddwd		mm3,[rdx+ofs_cbu]
	psrad		 mm6,14
	paddd		mm3,mm0
	pmaddwd	 mm7,[rdx+ofs_cbu]
	add	       rsi, BYTE 8
	add	       rdi, BYTE (12+4*(%2))
%if (%3==0)
       cmp	       rsi,rcx
%endif
	psrad		mm3,14		; mm3 = BBBBBBBBbbbbbbbb
	paddd		 mm7,mm4
	pxor		mm0,mm0
	psrad		 mm7,14
	packssdw	mm3,mm2	; mm3 = GGGGggggBBBBbbbb
	packssdw	 mm7,mm6
	packssdw	mm1,mm0	; mm1 = ________RRRRrrrr
	packssdw	 mm5,mm0	; *** avoid pxor mm4,mm4
	movq		mm2,mm3
	movq		 mm6,mm7
	punpcklwd	mm2,mm1	; mm2 = RRRRBBBBrrrrbbbb
	punpcklwd	 mm6,mm5
	punpckhwd	mm3,mm1	; mm3 = ____GGGG____gggg
	punpckhwd	 mm7,mm5
	movq		mm0,mm2
	movq		 mm4,mm6
	punpcklwd	mm0,mm3	; mm0 = ____rrrrggggbbbb
	punpcklwd	 mm4,mm7
%if (%2==0)
	psllq		mm0,16
	psllq		 mm4,16
%endif
	punpckhwd	mm2,mm3	; mm2 = ____RRRRGGGGBBBB
	punpckhwd	 mm6,mm7
	packuswb	mm0,mm2	; mm0 = __RRGGBB__rrggbb <- ta dah!
	packuswb	 mm4,mm6

%if %2
	por mm0, [rdx+ofs_xFF000000_FF000000]	 ; set alpha channels "on"
	por mm4, [rdx+ofs_xFF000000_FF000000]
	movq	[rdi-16],mm0	; store the quadwords independently
	movq	 [rdi-8],mm4
%else
	psrlq	mm0,8		; pack the two quadwords into 12 bytes
	psllq	mm4,8		; (note: the two shifts above leave
	movd	[rdi-12],mm0	; mm0,4 = __RRGGBBrrggbb__)
	psrlq	mm0,32
	por	mm4,mm0
	movd	[rdi-8],mm4
	psrlq	mm4,32
	movd	[rdi-4],mm4
%endif

%endmacro


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;void mmx_YUY2toRGB24(const uint8_t* src, uint8_t* dst, const uint8_t* src_end, int src_pitch, int row_size, int rec709);
;void mmx_YUY2toRGB32(const uint8_t* src, uint8_t* dst, const uint8_t* src_end, int src_pitch, int row_size, int rec709);

%macro YUV2RGB_PROC 2
	INIT_AMD64_TO_WIN64
	push	rsi
	push	rdi
	push	rbx
	
%ifdef WIN64
	%define src  (rsp + (1 + 3)*NPTR)
	movsxd rax, r9d
	mov rsi, r8
	mov rdi, rdx
	mov [src], rcx
%else
	%define src  (rsp + (1 + 3)*NPTR)
	mov	rax, [rsp + (4 + 3)*NPTR]		;src_pitch
	mov	rsi, [rsp + (3 + 3)*NPTR]		; read source bottom-up
	mov	rdi, [rsp + (2 + 3)*NPTR]
%endif

	movsxd rbx, DWORD [rsp + (5 + 3)*NPTR]
	movsxd rcx, dword [rsp + (6 + 3)*NPTR]

	test ecx, 1
	mov	rdx, yuv2rgb_constants_rec601
	jz .rec601
	mov	rdx, yuv2rgb_constants_rec601_full

.rec601:
.loop0:
	sub	rsi, rax
	lea	rcx, [rsi+rbx-8]

	align 32
.loop1:
	YUV2RGB_INNER_LOOP	%1, %2, 0
	jb	.loop1

	YUV2RGB_INNER_LOOP	%1, %2, 1

	sub	rsi, rbx
	
	cmp	rsi, [src]
	jne	.loop0

	pop	rbx
	pop	rdi
	pop	rsi
	DEINIT_AMD64_TO_WIN64	
	ret
	
%endmacro



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

cglobal mmx_YUY2toRGB24
ALIGN 16
mmx_YUY2toRGB24:
YUV2RGB_PROC 0, 0


cglobal mmx_YUY2toRGB32
ALIGN 16
mmx_YUY2toRGB32:
YUV2RGB_PROC 0, 1

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


