/******************************************************************************/
/*                                                                            */
/*  Cedocida DV-Codec                                                         */
/*                                                                            */
/*  Copyright (C) 2005 Andreas Dittrich <dittrich@cithraidt.de>               */
/*  Project start: 15.12.2003                                                 */
/*  Last change  : 22.05.2005                                                 */
/*                                                                            */
/******************************************************************************/
/*                                                                            */
/*  This program is free software; you can redistribute it and/or modify      */
/*  it under the terms of the GNU General Public License as published by      */
/*  the Free Software Foundation; either version 2 of the License, or         */
/*  (at your option) any later version.                                       */
/*                                                                            */
/*  This program is distributed in the hope that it will be useful,           */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of            */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             */
/*  GNU General Public License for more details.                              */
/*                                                                            */
/*  You should have received a copy of the GNU General Public License         */
/*  along with this program; if not, write to the Free Software               */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
/*                                                                            */
/******************************************************************************/

#include <string.h>

extern "C" { 

void emms_asm(void);
void byte_swap_block80_c(uint8_t *dst, uint8_t const *src, int N);
void byte_swap_block80_asm(uint8_t *dst, uint8_t const *src, int N);

void weighting_block_c(int16_t const *inp, int16_t *out, int16_t const *weight);
void weighting_block_MMX(int16_t const *inp, int16_t *out, int16_t const *weight);

void multiply_block_c(int16_t const *inp, int16_t *out, int16_t const *k);
void multiply_block_MMX(int16_t const *inp, int16_t *out, int16_t const *k);

void weighting_clipping_c(int16_t const *inp, int16_t *out, int16_t const *weight);
void weighting_clipping_MMX(int16_t const *inp, int16_t *out, int16_t const *weight);
void weighting_clipping_SSE(int16_t const *inp, int16_t *out, int16_t const *weight);

void coring_c(int16_t *block, int16_t const *core_positive_negative, int N);
void coring_MMX(int16_t *block, int16_t const *core_positive_negative, int N);

void copy_YUY2_to_MBYVU420_12bit_c(uint8_t const *inp, int16_t *out, int stride);
void copy_YUY2_to_MBYVU420_12bit_MMX(uint8_t const *inp, int16_t *out, int stride);

void copy_YUY2_to_MBYVU211_12bit_c(uint8_t const *inp, int16_t *out, int stride);

int is_interlaced_c(int16_t const *block, int threshold);
int is_interlaced_MMX(int16_t const *block, int threshold);

int max_abs_AC_c(int16_t const *block);
int max_abs_AC_MMX(int16_t const *block);


void clear_block128_c(uint8_t *blk, int N);
void clear_block128_asm(uint8_t *blk, int N);
void clear_block80_c(uint8_t *blk, int N);
void clear_block80_asm(uint8_t *blk, int N);
void ones_block80_c(uint8_t *blk, int N);
void ones_block80_asm(uint8_t *blk, int N);

void multiply_clip4095_MMX(int16_t const *inp, int16_t *out, int16_t const *k);
void multiply_clip_16bit_c(int16_t const *inp, int16_t *out, int16_t const *k);
void multiply_clip_16bit_MMX(int16_t const *inp, int16_t *out, int16_t const *k);

void multiply_clip_14bit(int16_t const *inp, int16_t *out, int16_t const *k);
void multiply_clip_14bit_MMX(int16_t const *inp, int16_t *out, int16_t const *k);

void copy_YV12_to_MBYVU420_12bit(
	uint8_t const *src_y, 
	uint8_t const *src_u, 
	uint8_t const *src_v, 
	int16_t *dst, 
	int stride_y, 
	int stride_uv);

void copy_YV12_to_MBYVU411border_12bit(
	uint8_t const *src_y, 
	uint8_t const *src_u, 
	uint8_t const *src_v, 
	int16_t *dst, 
	int stride_y, 
	int stride_uv);

void copy_YV12_to_MBYVU411_12bit(
	uint8_t const *src_y, 
	uint8_t const *src_u, 
	uint8_t const *src_v, 
	int16_t *dst, 
	int stride_y, 
	int stride_uv);

void copy_YUY2_to_MBYVU411border_12bit(uint8_t const *inp, int16_t *out, int stride);
void copy_YUY2_to_MBYVU411_12bit(uint8_t const *inp, int16_t *out, int stride);

}


#if 1
	#define weighting_block weighting_block_MMX
	#define multiply_block multiply_block_MMX
	#define multiply_clip_16bit multiply_clip_16bit_MMX
	#define weighting_clipping weighting_clipping_MMX
	#define coring coring_MMX
	#define is_interlaced is_interlaced_MMX
	#define max_abs_AC max_abs_AC_MMX
	#define copy_YUY2_to_MBYVU420_12bit copy_YUY2_to_MBYVU420_12bit_MMX
	#define clear_block80 clear_block80_asm
	#define clear_block128 clear_block128_asm
	#define ones_block80 ones_block80_asm
	#define byte_swap_block80 byte_swap_block80_asm
#else
	#define weighting_block weighting_block_c
	#define multiply_block multiply_block_c
	#define multiply_clip_16bit multiply_clip_16bit_c
	#define weighting_clipping weighting_clipping_c
	#define coring coring_c
	#define is_interlaced is_interlaced_c
	#define max_abs_AC max_abs_AC_c
	#define copy_YUY2_to_MBYVU420_12bit copy_YUY2_to_MBYVU420_12bit_c
	#define clear_block80 clear_block80_c
	#define clear_block128 clear_block128_c
	#define ones_block80 ones_block80_c
	#define byte_swap_block80 byte_swap_block80_c
#endif
