/******************************************************************************/
/*                                                                            */
/*  Cedocida DV-Codec                                                         */
/*                                                                            */
/*  Copyright (C) 2005 Andreas Dittrich <dittrich@cithraidt.de>               */
/*  Project start: 15.12.2003                                                 */
/*  Last change  : 15.06.2007                                                 */
/*                                                                            */
/******************************************************************************/
/*                                                                            */
/*  This program is free software; you can redistribute it and/or modify      */
/*  it under the terms of the GNU General Public License as published by      */
/*  the Free Software Foundation; either version 2 of the License, or         */
/*  (at your option) any later version.                                       */
/*                                                                            */
/*  This program is distributed in the hope that it will be useful,           */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of            */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             */
/*  GNU General Public License for more details.                              */
/*                                                                            */
/*  You should have received a copy of the GNU General Public License         */
/*  along with this program; if not, write to the Free Software               */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
/*                                                                            */
/******************************************************************************/


#include <string.h>

#include "../inttypes.h"

#include "../signalproc.h"
#include "bitstream.h"

#include "vlc.h"
#include "decode_vlc.h"
#include "decode_vlc_tables.h"




#define DECODE_VLC_GETNEXTBIT 0
#define DECODE_VLC_VALID 1
#define DECODE_VLC_EOB	  2
#define DECODE_VLC_ERROR 3

static int decode_vlc_risc( block_bitbuffer *blk_bb, uint32_t state[2], int16_t *block, int32_t mo )
{
	// this is a virtual risc machine doing the vlc-decoding
	// the function returns if the block is decoded or if no more bits are available
	// in the latter case, the decoding is continued at a consecutive call with the same state
	// there are four different instructions (see defines above)

	int *adress = (int*)(&state[0]);
	uint32_t *block_pos = &state[1];

	while (*adress>=0)
	{
		uint32_t code = decode_vlc_risc_program[*adress];
		uint32_t opcode = code >> 30;

		switch ( opcode ) {
		case DECODE_VLC_GETNEXTBIT:
			{
				int result = getbit( blk_bb );
				if (result<0)
					return *adress;	// no more bits are available, so return for now and try it later again
				else
				{
					uint32_t operand = (code>>16)&0x3FFF;
					*adress = operand + (uint32_t)result;
				}
				break;
			}
		case DECODE_VLC_VALID:
			{
				uint32_t run = (code>>16)&0x3FFF;
				int16_t amp = (int16_t)(code&0xFFFF);

				(*block_pos) += (run+1);
				if ( *block_pos<64 )
				{
					if (mo==0)
					{
						block[ scan_order_88[*block_pos] ] = amp;
					}
					else
					{
						block[ scan_order_248[*block_pos] ] = amp;
					}

					*adress = 0;
				}
				else	// should never happen in a regular DV-stream
					*adress = -1;

				break;
			}
		case DECODE_VLC_EOB:
			{
				*adress = -1;
				break;
			}
		case DECODE_VLC_ERROR:
			{
				// should not happen (only few codewords are not valid)
				*adress = -1;
				break;
			}
		}
	}

	return *adress;
}


decode_videosegment_func_t * decode_videosegment = decode_videosegment_ffmpeg;


extern "C" int decode_videosegment_cedocida( 
	int16_t *QTMB, 
	uint32_t *quantization_number, 
	uint32_t *DCT_mode, 
	uint32_t *class_number, 
	uint8_t const *DIF_block_in)
{
	block_bitbuffer blk_bb[5*6];
	uint32_t blk_state[5*6*2];
	uint8_t DIF_block[5*80];

	int j;
	int n = 0;
	int16_t *block = QTMB;

	byte_swap_block80(DIF_block, DIF_block_in, 5);

	init_bitbuffer_decode( blk_bb, DIF_block );
	memset(blk_state, 0x00, sizeof(blk_state));

	clear_block128( (uint8_t*)block, 5*6 );

	for (j=0; j<5; j++)
	{
		int l;
		
		uint32_t SCT_Seq_Dseq_DBN_STA_QNO = ((uint32_t*)DIF_block)[j*20];

		quantization_number[j] = SCT_Seq_Dseq_DBN_STA_QNO&0x0F;
		
		for (l=0;l<6;l++)
		{
			uint32_t DCI_mo_c1_c0;

			if ((blk_bb+n)->modulo==0)
				DCI_mo_c1_c0 = (blk_bb+n)->pos[0]&0xFFF00000;
			else
				DCI_mo_c1_c0 = ((blk_bb+n)->pos[0]<<16)&0xFFF00000;

			(blk_bb+n)->modulo += 12;
			(blk_bb+n)->level -= 12;

			block[0] = ((int32_t)DCI_mo_c1_c0)>>23;
			DCT_mode[n] = (DCI_mo_c1_c0&0x00400000)>>22;
			class_number[n] = (DCI_mo_c1_c0&0x00300000)>>20;

			decode_vlc_risc( blk_bb+n, blk_state+2*n, block, DCT_mode[n] );

			block += 64;
			n++;
		}

		for (l=0;l<6;l++)
		{
			if( decode_vlc_risc( blk_bb+n-6+0, blk_state+2*(n-6)+2*l, block + 64*(l-6), DCT_mode[n-6+l] ) >= 0 )
			{
				decode_vlc_risc( blk_bb+n-6+1, blk_state+2*(n-6)+2*l, block + 64*(l-6), DCT_mode[n-6+l] );
				decode_vlc_risc( blk_bb+n-6+2, blk_state+2*(n-6)+2*l, block + 64*(l-6), DCT_mode[n-6+l] );
				decode_vlc_risc( blk_bb+n-6+3, blk_state+2*(n-6)+2*l, block + 64*(l-6), DCT_mode[n-6+l] );
				decode_vlc_risc( blk_bb+n-6+4, blk_state+2*(n-6)+2*l, block + 64*(l-6), DCT_mode[n-6+l] );
				decode_vlc_risc( blk_bb+n-6+5, blk_state+2*(n-6)+2*l, block + 64*(l-6), DCT_mode[n-6+l] );
			}
		}

	}

	block = block - 64*5*6;
	for (n=0; n<(5*6); n++)
	{
		for (j=0;j<(5*6);j++)
		{
			if( decode_vlc_risc( blk_bb+j, blk_state+2*n, block, DCT_mode[n] ) < 0 ) break;
		}

		block += 64;
	}

	return 0;
}

